/*
 * regexp.c: Regular expression list
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#include "regexp.h"
#include <unistd.h>

/* Global instance */
cEpgfixerRegexps EpgfixerRegexps;

const char *strSources[] = { "title","shorttext","description","undefined" };

cRegexp::cRegexp()
{
  enabled = false;
  error = NULL;
  erroffset = -1;
  regexp = NULL;
  source = REGEXP_UNDEFINED;
  re = NULL;
  sd = NULL;
}

cRegexp::cRegexp(int Source, const char *Regex, bool Enabled, bool Precompile)
{
  enabled = Enabled;
  error = NULL;
  erroffset = -1;
  regexp = strdup(Regex);
  source = Source;
  re = NULL;
  sd = NULL;
  if (Precompile)
     Compile();
}

cRegexp::~cRegexp(void)
{
  free(regexp);
  FreeCompiled();
}

void cRegexp::SetRegexp(const char *Regex)
{
  free(regexp);
  regexp = strdup(Regex);
}

void cRegexp::ToggleEnabled(void)
{
  enabled = enabled ? 0 : 1;
}

void cRegexp::SetFromString(char *string, bool Enabled, bool Precompile)
{
  free(regexp);
  enabled = Enabled;
  char *s = string;
  bool precompile = Precompile;
  if (s[0] == '!' || s[0] == '#') {
     enabled = false;
     precompile = false;
     }
  char *p = (s[0] == '#') ? s : strchr(s, '=');
  if (p) {
     char *regex;
     if (p[0] == '#')
        regex = p;
     else {
        *p = 0;
        regex = p + 1;
        }
     char *field = (s[0] == '!') ? s+1 : s;
     source = REGEXP_UNDEFINED;
     if (strcmp(field, "title") == 0)
        source = REGEXP_TITLE;
     if (strcmp(field, "shorttext") == 0)
        source = REGEXP_SHORTTEXT;
     if (strcmp(field, "description") == 0)
        source = REGEXP_DESCRIPTION;
     regexp = strdup(regex);
     if (precompile)
        Compile();
     }
}

void cRegexp::FreeCompiled()
{
  if (re) {
     pcre_free(re);
     re = NULL;
     }
  if (sd) {
#ifdef PCRE_CONFIG_JIT
     pcre_free_study(sd);
#else
     pcre_free(sd);
#endif
     sd = NULL;
     }
}

void cRegexp::Compile()
{
  FreeCompiled();
  re = pcre_compile(regexp, 0, (const char **)&error, &erroffset, NULL);
  if (error) {
     esyslog("PCRE compile error: %s at offset %i", error, erroffset);
     enabled = false;
     }
  else {
     sd = pcre_study(re, 0, (const char **)&error);
     if (error)
        esyslog("PCRE study error: %s", error);
     }
}

cEpgfixerRegexps::cEpgfixerRegexps()
{
  fileName = NULL;
  regexps = new cRegexpList();
}

cEpgfixerRegexps::~cEpgfixerRegexps()
{
  free(fileName);
  delete regexps;
}

void cEpgfixerRegexps::SetRegexpConfigFile(const char *FileName)
{
  fileName = strdup(FileName);
}
const char *cEpgfixerRegexps::RegexpConfigFile()
{
  return fileName;
}

bool cEpgfixerRegexps::ReloadRegexps(bool AllowComments, bool Precompile)
{
  regexps->Clear();
  return LoadRegexps(fileName, AllowComments, Precompile);
}

bool cEpgfixerRegexps::LoadRegexps(const char *FileName, bool AllowComments, bool Precompile)
{
  bool result = false;
  if (FileName && access(FileName, F_OK) == 0) {
     FILE *f = fopen(FileName, "r");
     if (f) {
        char *s;
        int line = 0;
        cReadLine ReadLine;
        while ((s = ReadLine.Read(f)) != NULL) {
              line++;
              if (!isempty(s)) {
                 regexps->Add(new cRegexp());
                 regexps->Last()->SetFromString(s, true, Precompile);
                 }
              }
        fclose(f);
        }
     else {
        LOG_ERROR_STR(FileName);
        result = false;
        }
     }
  return result;
}
